#!/usr/bin/env python3
import random
import time
import sys
import curses
import argparse
from tscrnsvr.shared import ScreenSpace, print_error, parse_chars

__VERSION__ = "v0.1.1"

# matrix.py
# A cool matrix effect for your terminal!
# Fully customizable, of course.
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0

class Fade:
	def __init__(self,starttick,x,y,headcolor,colors,unics,length,lifetime,speed):
		self.st		= starttick 
		self.colors	= colors
		self.lcolors= len(self.colors)
		self.headcolor = headcolor
		self.unics	= unics
		self.length = length
		self.lt		= lifetime
		self.speed	= speed
		self.pos	= [x,y]
		self.time	= 0
		self.dstd	= 0
	def render(self,tick,screenspace):
		self.time += 1
		for i in range(self.length):
			if i == 0:
				self.pendingcolor = self.headcolor
			else:
				self.pendingcolor = self.colors[int((i/self.length)*self.lcolors)]
			self.pendingchar  = random.choice(self.unics)
			screenspace.addchar(self.pos[0],round(self.pos[1])-i,self.pendingchar, \
					self.pendingcolor)
		self.pos[1] += self.speed
		if self.time == round(self.lt):
			self.dstd = 1
	def destroy(self):
		self = None
		
def main(stdscr,params,chars):
	STARTTIME = time.time()
	PARAMS = params
	FADES	= [None for fd in range(PARAMS.fades)]
	TICK = 0
	SCSP = ScreenSpace(stdscr)
	for fade in range(PARAMS.fades):
		FADES[fade] = Fade(
				TICK,
				random.randint(0,SCSP.size[1]),
				random.randint(0,SCSP.size[0]),
				PARAMS.headcolor,
				[int(j) for j in PARAMS.colors.rstrip(",").split(",")],
				chars,
				random.randint(PARAMS.minlength,PARAMS.maxlength),
				random.uniform(PARAMS.minlifetime,PARAMS.maxlifetime),
				random.uniform(PARAMS.minspeed,PARAMS.maxspeed))
	try:
		while True:
			time.sleep(PARAMS.delay)
			for fade in range(PARAMS.fades):
				if FADES[fade].dstd == 1:
					FADES[fade] = Fade(
				TICK,
				random.randint(0,SCSP.size[1]),
				random.randint(0,SCSP.size[0]),
				PARAMS.headcolor,
				[int(j) for j in PARAMS.colors.rstrip(",").split(",")],
				chars,
				random.randint(PARAMS.minlength,PARAMS.maxlength),
				random.uniform(PARAMS.minlifetime,PARAMS.maxlifetime),
				random.uniform(PARAMS.minspeed,PARAMS.maxspeed))
			for fade in FADES:
				fade.render(TICK,SCSP)
			if PARAMS.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not PARAMS.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if PARAMS.debug == True:
				deb.append("matrix.py %s" % __VERSION__)
				deb.append("%s\tframes, delay %s, colors %s, fades %s" % (str(TICK),str(PARAMS.delay),PARAMS.colors,str(PARAMS.fades)))
			SCSP.render(deb)
			TICK += 1
	except KeyboardInterrupt:
		SCSP.destroy()
		sys.exit()

def get_args():
	return [
			"matrix",
			"A screensaver like the one in the movie \"The Matrix\"",
			
			[(("-d", "--delay"), {"type":float, "default":0.1,
				"help":"set delay between frames in seconds (default 0.1)"}),
			
			(("-n", "--fades"), {"type":int, "default":40,
				"help":"set number of \"fades\" on screen (default 25)"}),
			
			(("-miln", "--minlength"), {"type":int, "default":4,
				"help":"set minimum length of fades (default 4)"}),
			
			(("-mxln", "--maxlength"), {"type":int, "default":20,
				"help":"set maximum length of fades (default 20)"}),
			
			(("-mis", "--minspeed"), {"type":float, "default":0.5,
				"help":"set minimum speed of fades (default 0.5)"}),
			
			(("-mxs", "--maxspeed"), {"type":float, "default":2,
				"help":"set maximum speed of fades (default 2)"}),
			
			(("-milt", "--minlifetime"), {"type":int, "default":10,
				"help":"set minimum lifetime of fades (default 10)"}),
			
			(("-mxlt", "--maxlifetime"), {"type":int, "default":30,
				"help":"set maximum lifetime of fades (default 30)"}),
			
			(("-c", "--colors"), {"type":str, "default":"83,84,119,120",
				"help":"set fade colors.\n"+\
				"Must be a comma separated list of values on the terminal's color palette.\n"+\
				"(default 83,84,119,120)"}),
			
			(("-cc", "--headcolor"), {"type":int, "default":16,
				"help":"set color of the first character of every fade."}),
			
			(("-C", "--chars"), {"type":str, "default":"32-128",
				"help":"set fade characters. Is a comma separated list of unicode ranges.\n"+\
					"Can be one character or a range. [Example: -C 32-55,66,2605,2606,6785-8941]\n"+\
					"(default 32-128)"}),
			
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"screensaver mode (press any key to leave)"})],
			]

def run(args):
	chars = parse_chars(args.chars)
	curses.wrapper(main,args,chars)


#!/usr/bin/env python3
import argparse
import curses
import importlib

COMMANDS = sorted([
			"matrix",
			"pipes",
			"bounce",
			"colortest",
			"clock",
			"imgbounce",
			"snake",
			"stars"
			])

def main():
	par = argparse.ArgumentParser(prog='tscrnsvr')
	mainsubpar = par.add_subparsers()

	subparsers = {}

	for cmd in COMMANDS:
		module = importlib.import_module("tscrnsvr."+cmd)
		args = module.get_args()
		subparsers[cmd] = mainsubpar.add_parser(args[0],help=args[1])
		if args[2] != None:
			args[2] = sorted(args[2],key=lambda x: x[0][0].lower())
			for argn in args[2]:
				subparsers[cmd].add_argument(*argn[0],**argn[1])
		subparsers[cmd].set_defaults(function=module.run)

	ags = par.parse_args()
	if hasattr(ags,"function"):
		ags.function(ags)
	else:
		par.print_help()


#!/usr/bin/env python3
import random
import time
import sys
import curses
import argparse
import math
from tscrnsvr.shared import ScreenSpace, print_error, get_center, parse_chars

__VERSION__ = "v0.1.0"

# stars.py
# Star screensaver.
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0
	
class Star:
	def __init__(self, color, x, y, acceleration, direction, char):
		self.pos = [x,y]
		self.color = color
		self.acceleration = acceleration
		self.direction = direction
		self.dstd = False
		self.char = char
	def render(self,screenspace):
		screenspace.addchar(round(self.pos[0]),round(self.pos[1]),self.char,self.color)
		self.pos[0] += self.direction[0] * self.acceleration
		self.pos[1] += self.direction[1] * self.acceleration
		if not screenspace.in_bounds(self.pos[0],self.pos[1]):
			self.dstd = True
	def destroy(self):
		self = None

def main(stdscr, PARAMS, chars):
	SCSP = ScreenSpace(stdscr)
	STARTTIME = time.time()
	TICK = 0
	clist = [int(i) for i in PARAMS.colors.split(',')]
	stlist = []
	try:
		while True:
			if random.random() < PARAMS.chance:
				ang = random.randint(0,359)
				stlist.append(Star(random.choice(clist),*get_center(stdscr),PARAMS.acceleration,[math.cos(ang),math.sin(ang)],random.choice(chars)))
			for n, star in enumerate(stlist):
				if star.dstd:
					star.destroy()
					stlist.pop(n)
				else:
					star.render(SCSP)
			if PARAMS.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not PARAMS.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if PARAMS.debug == True:
				deb.append("stars.py %s" % __VERSION__)
				deb.append("%s\tframes, delay %s, stars %s" % (str(TICK),str(PARAMS.delay),str(len(stlist))))
			SCSP.render(deb)
			TICK += 1
			time.sleep(PARAMS.delay)
	except KeyboardInterrupt:
		sys.exit()
	
def get_args():
	return [
			"stars",
			"Make \"stars\" fly across your screen.",

			[(("-d", "--delay"), {"type":float, "default":0.01,
				"help":"set delay between frames in seconds (default 0.01)"}),
				
			(("-q", "--chance"), {"type":float, "default":0.2,
				"help":"Chance for a star to spawn each frame (default 0.2)"}),
				
			(("-c", "--colors"), {"type":str, "default":"227",
				"help": "set star colors.\n"+\
				"Must be a comma separated list of values on the terminal's color palette.\n"+\
				"(default 227)"}),
			
			(("-C", "--chars"), {"type":str, "default":"42",
				"help":"set star characters. Is a comma separated list of unicode ranges.\n"+\
					"Can be one character or a range. [Example: -C 32-55,66,2605,2606,6785-8941]\n"+\
					"(default 42)"}),
					
			(("-a", "--acceleration"), {"type":float, "default":0.3,
				"help":"Acceleration rate of stars (default 0.3)"}),
			
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"screensaver mode (press any key to leave)"})],
			]

def run(args):
	chars = parse_chars(args.chars)
	curses.wrapper(main,args,chars)



#!/usr/bin/env python3
import random
import time
import sys
import curses
import argparse
import math
from tscrnsvr.shared import ScreenSpace

__VERSION__ = "v0.1"

# pipes.py
# Pipe screensaver 
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0

PIPETYPES =	[
		"│╭X╮╯─╮XX╰│╯╰X╭─",
		"│┌X┐┘─┐XX└│┘└X┌─",
		"┃┏X┓┛━┓XX┗┃┛┗X┏━",
		"║╔X╗╝═╗XX╚║╝╚X╔═"
		]

class Pipe:
	def __init__(self,color,screenspace,style):
		self.color 	= int(color)
		self.dirch 	= [[0,-1],
						[1,0],
						[0,1],
						[-1,0]]
		self.dir	= random.randint(0,3)
		self.realdir= self.dirch[self.dir]
		self.pos	= [random.randint(0,screenspace.size[1]),random.randint(0,screenspace.size[0])]
		self.chars 	= style
	def render(self,screenspace):
		if random.random() <= 0.20:
			self.ldir = self.dir
			self.choose = True
			while True:
				self.dir = random.randint(0,3)
				if abs(self.dirch[self.dir][0]) != abs(self.dirch[self.ldir][0]):
					if abs(self.dirch[self.dir][1]) != abs(self.dirch[self.ldir][1]):
						break
			self.realdir = self.dirch[self.dir]
			self.char = self.chars[(self.ldir*4)+self.dir]
		else:
			self.char = self.chars[((self.dir)*5)]
		screenspace.addchar(self.pos[0] % screenspace.size[1],self.pos[1] % screenspace.size[0],ord(self.char),self.color)
		self.pos[0] += self.realdir[0]
		self.pos[1] += self.realdir[1]

def main(stdscr,args):
	SCSP = ScreenSpace(stdscr)
	PIPES = []
	STARTTIME = time.time()
	while True:
		TICK = 0
		for pp in range(0,args.pipes):
			PIPES.append(Pipe(random.choice(args.colors.split(",")),
						SCSP,
						PIPETYPES[args.style-1]))
		for fr in range(0,args.rstime):
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not args.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if args.debug == True:
				deb.append("pipes.py %s" % __VERSION__)
				deb.append("%s\tframes, delay %s, pipes %s" % (str(TICK),str(args.delay),args.pipes))
			if args.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			TICK += 1
			for pp in PIPES:
				pp.render(SCSP)
			SCSP.render(deb,clear=False)
			time.sleep(args.delay)
		PIPES = []
		SCSP.win.clear()

def get_args():
	return [
			"pipes",
			"Screensaver similar to that of Windows XP; only 2D.",
			
			[(("-d", "--delay"), {"type":float, "default":0.1,
				"help":"set delay between frames in seconds (default 0.1)"}),
			
			(("-n", "--pipes"), {"type":int, "default":10,
				"help":"set number of pipes on screen (default 10)"}),
			
			(("-c", "--colors"), {"type":str, "default":",".join([str(x) for x in range(0,256)]),
				"help":"set pipe colors. "+\
					"Must be a comma separated list of values on the terminal's color palette. "+\
					"(default random choice)"}),
			
			(("-C", "--style"), {"type":int, "default":3,
				"help":"set pipe style. Non-zero integer up to 4."}),
			
			(("-r", "--rstime"), {"type":int, "default":200,
				"help":"restart pipe system every RSTIME frames. (default 200)"}),
				
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"screensaver mode (press any key to leave)"})]
			]

def run(args):
	curses.wrapper(main,args)


#!/usr/bin/env python3
import random
import time
import curses
import argparse
import sys
from tscrnsvr.shared import ScreenSpace, print_error, ColorObj
try:
	from PIL import Image
except:
	print_error("WARNING: The imgbounce module requires Pillow!")

__VERSION__ = "v0.1.1"

# imgbounce.py
# Bounce image screensaver.
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0

class ImgBounceText:
	def __init__(self,win,img,args):
		self.win	= win
		self.pos 	= [10,10]
		self.co  	= ColorObj()
		self.colmap = []
		self.size 	= [int(c) for c in args.dimensions.split("x")]
		self.bbsize = self.size
		try:
			self.img = Image.open(img)
			self.img = self.img.convert("RGB")
			self.img = self.img.resize(self.size)
		except:
			print_error("IMGBOUNCE ERROR: Could not load image!")
			sys.exit(2)
		self.motion = [random.choice([-1,1]),random.choice([-1,1])]
		for y in range(self.size[1]):
			for x in range(self.size[0]):
				pc = self.img.getpixel((x,y))
				apxc = self.co.approxColor(pc)
				self.colmap.append(apxc)
	def render(self,screenspace):
		for y in range(self.size[1]):
			for x in range(self.size[0]):
				screenspace.addchar(self.pos[0]+(x*2),self.pos[1]+y,ord("█"),self.colmap[(y*self.size[1])+x])
				screenspace.addchar(self.pos[0]+(x*2)-1,self.pos[1]+y,ord("█"),self.colmap[(y*self.size[1])+x])
		for xu in range(self.pos[0]-1, \
						self.pos[0]+(self.size[0])*2+1):
			for yu in range(self.pos[1]-1, \
							self.pos[1]+self.size[1]+1):
				if xu < 0:
					self.motion[0] = 1
				if xu >= self.win.getmaxyx()[1]:
					self.motion[0] = -1
				if yu < 0:
					self.motion[1] = 1
				if yu >= self.win.getmaxyx()[0]:
					self.motion[1] = -1
		self.pos[0] += self.motion[0]
		self.pos[1] += self.motion[1]
	
def main(stdscr,args):
	STARTTIME = time.time()
	TICK = 0
	SCSP = ScreenSpace(stdscr)
	bt	 = ImgBounceText(stdscr,args.image,args)
	try:
		while True:
			bt.render(SCSP)
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not args.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if args.debug == True:
				deb.append("bounce.py %s" % __VERSION__)
				deb.append("%s\tframes, delay %s, image %s" % (str(TICK),str(args.delay),args.image))
			SCSP.render(debug=deb,ref=False,clear=False)
			time.sleep(args.delay)
			if args.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			SCSP.win.erase()
			TICK += 1
	except KeyboardInterrupt:
		SCSP.destroy()
		sys.exit()

def get_args():
	return [
			"imgbounce",
			"Bounce an image around your terminal.",
			
			[(("-d", "--delay"), {"type":float, "default":0.1,
				"help":"set delay between frames in seconds (default 0.1)"}),
			
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"screensaver mode (press any key to leave)"}),
				
			(("-x", "--dimensions"), {"type":str, "default":"32x32",
				"help":"dimensions for image to bounce (default 32x32)"}),
			
			(["image"], {"type":str,
				"help":"set image for bounce"})]
			]

def run(args):
	curses.wrapper(main,args)
		


#!/usr/bin/env python3
import curses

def colortest(stdscr):
	# http://stackoverflow.com/questions/18551558/how-to-use-terminal-color-palette-with-curses
	curses.start_color()
	curses.use_default_colors()
	stdscr.addstr(0,0,"%s colors on \"%s\"\n" % (str(curses.COLORS),str(curses.termname().decode('ascii'))))
	for i in range(0, curses.COLORS):
		curses.init_pair(i + 1, i, -1)
	for i in range(curses.COLORS):
		try:
			stdscr.addstr(str(i)+"\t",curses.color_pair(i))
		except:
			pass
	try:
		stdscr.getch()
	except: pass
def get_args():
	return [
			"colortest",
			"Displays color palette.",
			None
			]

def run(args):
	curses.wrapper(colortest)


import curses
import base64
import math
import time

class ScreenSpace:
	def __init__(self,win):
		self.win	= win
		self.size	= self.win.getmaxyx()
		curses.start_color()
		curses.use_default_colors()
		self.win.nodelay(1)
		for i in range(0, curses.COLORS):
			curses.init_pair(i + 1, i, -1)
		curses.curs_set(0)
	def addchar(self,x,y,char,mods):
		self.size = self.win.getmaxyx()
		#print(x,y,char,mods)
		try:
			self.win.addstr(int(y),int(x),chr(char),curses.color_pair(mods))
		except:
			pass
	def addstr(self,x,y,str,mods):
		self.size = self.win.getmaxyx()
		try:
			self.win.addstr(int(y),int(x),str,curses.color_pair(mods))
		except:
			pass
	def render(self,debug=None,clear=True,ref=True):
		self.size = self.win.getmaxyx()
		if debug != None:
			for dstr in range(len(debug)):
				self.win.addstr(dstr,0,debug[dstr],curses.color_pair(0))
		if ref == True:
			self.win.refresh()
		if clear == True:
			self.win.erase()
	def destroy(self):
		curses.nocbreak()
		self.win.keypad(0)
		curses.echo()
		curses.endwin()
		curses.curs_set(1)
	def in_bounds(self, x, y):
		self.size = self.win.getmaxyx()
		if y >= 0:
			if y < self.size[0]:
				if x >= 0:
					if x < self.size[1]:
						return True
		return False

class ColorObj:
	def __init__(self):
		self.colorstr = [ 	
			"AAAAwoAAAADCgADCgMKAAAAAwoDCgADCgADCgMKAw4DDgMOAwoDCgMKAw78A",
			"AADDvwDDv8O/AAAAw7/DvwDDvwDDv8O/w78Aw7/Dv8O/w78AAAAAAF8AAMKH",
			"AADCrwAAw5cAAMO/AF8AAF9fAF/ChwBfwq8AX8OXAF/DvwDChwAAwodfAMKH",
			"wocAwofCrwDCh8OXAMKHw78Awq8AAMKvXwDCr8KHAMKvwq8Awq/DlwDCr8O/",
			"AMOXAADDl18Aw5fChwDDl8KvAMOXw5cAw5fDvwDDvwAAw79fAMO/wocAw7/C",
			"rwDDv8OXAMO/w79fAABfAF9fAMKHXwDCr18Aw5dfAMO/X18AX19fX1/Ch19f",
			"wq9fX8OXX1/Dv1/ChwBfwodfX8KHwodfwofCr1/Ch8OXX8KHw79fwq8AX8Kv",
			"X1/Cr8KHX8Kvwq9fwq/Dl1/Cr8O/X8OXAF/Dl19fw5fCh1/Dl8KvX8OXw5df",
			"w5fDv1/DvwBfw79fX8O/wodfw7/Cr1/Dv8OXX8O/w7/ChwAAwocAX8KHAMKH",
			"wocAwq/ChwDDl8KHAMO/wodfAMKHX1/Ch1/Ch8KHX8Kvwodfw5fCh1/Dv8KH",
			"wocAwofCh1/Ch8KHwofCh8KHwq/Ch8KHw5fCh8KHw7/Ch8KvAMKHwq9fwofC",
			"r8KHwofCr8KvwofCr8OXwofCr8O/wofDlwDCh8OXX8KHw5fCh8KHw5fCr8KH",
			"w5fDl8KHw5fDv8KHw78AwofDv1/Ch8O/wofCh8O/wq/Ch8O/w5fCh8O/w7/C",
			"rwAAwq8AX8KvAMKHwq8Awq/CrwDDl8KvAMO/wq9fAMKvX1/Cr1/Ch8KvX8Kv",
			"wq9fw5fCr1/Dv8KvwocAwq/Ch1/Cr8KHwofCr8KHwq/Cr8KHw5fCr8KHw7/C",
			"r8KvAMKvwq9fwq/Cr8KHwq/Cr8Kvwq/Cr8OXwq/Cr8O/wq/DlwDCr8OXX8Kv",
			"w5fCh8Kvw5fCr8Kvw5fDl8Kvw5fDv8Kvw78Awq/Dv1/Cr8O/wofCr8O/wq/C",
			"r8O/w5fCr8O/w7/DlwAAw5cAX8OXAMKHw5cAwq/DlwDDl8OXAMO/w5dfAMOX",
			"X1/Dl1/Ch8OXX8Kvw5dfw5fDl1/Dv8OXwocAw5fCh1/Dl8KHwofDl8KHwq/D",
			"l8KHw5fDl8KHw7/Dl8KvAMOXwq9fw5fCr8KHw5fCr8Kvw5fCr8OXw5fCr8O/",
			"w5fDlwDDl8OXX8OXw5fCh8OXw5fCr8OXw5fDl8OXw5fDv8OXw78Aw5fDv1/D",
			"l8O/wofDl8O/wq/Dl8O/w5fDl8O/w7/DvwAAw78AX8O/AMKHw78Awq/DvwDD",
			"l8O/AMO/w79fAMO/X1/Dv1/Ch8O/X8Kvw79fw5fDv1/Dv8O/wocAw7/Ch1/D",
			"v8KHwofDv8KHwq/Dv8KHw5fDv8KHw7/Dv8KvAMO/wq9fw7/Cr8KHw7/Cr8Kv",
			"w7/Cr8OXw7/Cr8O/w7/DlwDDv8OXX8O/w5fCh8O/w5fCr8O/w5fDl8O/w5fD",
			"v8O/w78Aw7/Dv1/Dv8O/wofDv8O/wq/Dv8O/w5fDv8O/w78ICAgSEhIcHBwm",
			"JiYwMDA6OjpEREROTk5YWFhgYGBmZmZ2dnbCgMKAwoDCisKKworClMKUwpTC",
			"nsKewp7CqMKowqjCssKywrLCvMK8wrzDhsOGw4bDkMOQw5DDmsOaw5rDpMOk",
			"w6TDrsOuw64=" ]
		self.rcolors = [ord(c) for c in base64.b64decode(''.join(self.colorstr)).decode("utf-8")]
		self.colors = {}
		self.cn = 0
		for a, b, c in zip(*[iter(self.rcolors)]*3): self.colors[self.cn] = ((a,b,c)); self.cn += 1
		self.rcolors = {v: k for k, v in self.colors.items()}

	def colorDistance(self, c1, c2):
		try:
			(r1,g1,b1) = c1
			(r2,g2,b2) = c2
			return math.sqrt((r1 - r2)**2 + (g1 - g2) ** 2 + (b1 - b2) **2)
		except:
			return -1

	def approxColor(self,cl):
		closest_colors = sorted(self.rcolors, key=lambda color: self.colorDistance(color, cl))
		return self.rcolors[closest_colors[0]]

def print_error(s):
	print("\x1b[31m%s\x1b[0m" % s)

def get_center(win):
	dims = win.getmaxyx()
	return [round(dims[1]/2),round(dims[0]/2)]
	
def parse_chars(chrl):
	charlist = []
	try:
		chrl = chrl.split(",")
		for chrr in chrl:
			if "-" in chrr:
				dashs = chrr.split("-")
				if len(dashs) == 2:
					charlist.extend(range(int(dashs[0]),int(dashs[1])))
			elif chrr != "":
				charlist.append(int(chrr))
			assert len(charlist) > 0
		return charlist
	except:
		print_error("MATRIX ERROR: Could not parse characters list!")
		sys.exit(1)
	


#!/usr/bin/env python3
import curses
import time
import random
import sys
from tscrnsvr.shared import ScreenSpace, get_center

def wrap(point,size):
	return [point[0] % size[1], point[1] % size[0]]

class Snake:
	def __init__(self,win,args):
		self.pos = get_center(win)
		self.wsize = win.getmaxyx()
		self.dir = [0,1]
		self.length = args.startlength
		self.tailpos = []
		self.fruitpos = [random.randint(0,self.wsize[1]-1),
						random.randint(0,self.wsize[0]-1)]
		for g in range(1,self.length+1):
			tp = self.pos[:]
			tp[1] = tp[1] - g
			self.tailpos.append(tp)
	def turnleft(self):
		if self.dir[1] == 1:
			return [1,0]
		elif self.dir[1] == -1:
			return [-1,0]
		elif self.dir[0] == 1:
			return [0,-1]
		elif self.dir[0] == -1:
			return [0,1]
	def turnright(self):
		if self.dir[1] == 1:
			return [-1,0]
		elif self.dir[1] == -1:
			return [1,0]
		elif self.dir[0] == 1:
			return [0,1]
		elif self.dir[0] == -1:
			return [0,-1]
	def render(self,screenspace,args):
		wsp = wrap(self.pos,screenspace.win.getmaxyx())
		screenspace.addchar(wsp[0],wsp[1],args.char,args.color)
		for ch in self.tailpos:
			wch = wrap(ch,screenspace.win.getmaxyx())
			screenspace.addchar(wch[0],wch[1],args.char,args.color)
		screenspace.addchar(self.fruitpos[0],self.fruitpos[1],args.fruitchar,args.fruitcolor)
		if wrap(self.pos,screenspace.win.getmaxyx()) != self.fruitpos:
			del self.tailpos[-1]
		else:
			self.wsize = screenspace.win.getmaxyx()
			self.fruitpos = [random.randint(0,self.wsize[1]-1),
							random.randint(0,self.wsize[0]-1)]
		self.tailpos.insert(0,self.pos[:])
		self.pos[0] += self.dir[0]
		self.pos[1] += self.dir[1]
		if wrap(self.pos,screenspace.win.getmaxyx()) in self.tailpos:
			screenspace.destroy()
			sys.exit()
		
def main(win,args):
	SCSP = ScreenSpace(win)
	SNK = Snake(win,args)
	try:
		while True:
			SNK.render(SCSP,args)
			SCSP.render(clear=False,ref=False)
			gc = SCSP.win.getch()
			if gc in [ord("a"),curses.KEY_LEFT]:
				SNK.dir = SNK.turnleft()
			if gc in [ord("d"),curses.KEY_RIGHT]:
				SNK.dir = SNK.turnright()
			if gc == ord("q"):
				SCSP.destroy()
				sys.exit()
			SCSP.win.erase()
			time.sleep(args.delay)
	except KeyboardInterrupt:
		pass

def get_args():
	return [
			"snake",
			"Play snake!",
			
			[(("-C", "--char"), {"type":int, "default":64,
				"help":"set code of the character for the snake's head and body (default 64)"}),
				
			(("-c", "--color"), {"type":int, "default":209,
				"help":"set color for snake"}),
			
			(("-F", "--fruitchar"), {"type":int, "default":65,
				"help":"set character for fruit (default 65)"}),
			
			(("-f", "--fruitcolor"), {"type":int, "default":125,
				"help":"set color for fruit (default 125)"}),
			
			(("-l", "--startlength"), {"type":int, "default":5,
				"help":"set starting length of snake"}),
			
			(("-d", "--delay"), {"type":float, "default":0.1,
				"help":"delay between each frame (default 0.1)"}),
			
			]]

def run(args):
	curses.wrapper(main,args)


from setuptools import setup, find_packages

setup(name="tscrnsvr",
    version="0.1.0a",
    description="Cool screensavers for the terminal",
    long_description="""`Repo
<https://github.com/illinoisjackson/tscrnsvr/>`_""",
    url="https://github.com/illinoisjackson/terminal-screensavers",
    author="illinoisjackson",
    author_email="icantpostmyemailhere@gmail.com",
    license="MIT License",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3 :: Only"
    ],
    keywords="screensaver terminal",
    packages=find_packages(exclude=["images"]),
	entry_points = {
        'console_scripts': ['tscrnsvr=tscrnsvr.cmdline:main'],
    }
)


#!/usr/bin/env python3
import argparse
import curses
import time
import subprocess
import sys
from tscrnsvr.shared import ScreenSpace

__VERSION__ = "v0.1"

# clock.py
# clock screensaver 
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0

def main(win,args):
	STARTTIME = time.time()
	TICK = 0
	SCSP = ScreenSpace(win)
	try:
		while True:
			formattedfigtime = time.strftime(args.topstrf)
			formattedsmalltime = time.strftime(args.bottomstrf)
			figtext = subprocess.check_output(["figlet","-f",args.font,formattedfigtime]).splitlines()
			ll = len(max(figtext, key=len))
			fh = len(figtext)
			sx = round((SCSP.size[1]-ll)/2)-1
			sy = round((SCSP.size[0]-fh)/2)
			for line,ln in enumerate(figtext):
				SCSP.addstr(sx,sy+line,ln,args.topcolor)
			SCSP.addstr(round((SCSP.size[1]-len(formattedsmalltime))/2),fh+1+sy,formattedsmalltime,args.bottomcolor)
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not args.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if args.debug == True:
				deb.append("clock.py %s" % __VERSION__)
				deb.append("%s\tframes" % (str(TICK)))
			SCSP.render(debug=deb,ref=False,clear=False)
			if args.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			SCSP.win.erase()
			time.sleep(0.1)
			TICK += 1
	except KeyboardInterrupt:
		sys.exit()

def get_args():
	return [
			"clock",
			"A simple digital clock. Requires FIGlet to be installed.",
			
			[(("-f", "--font"), {"type":str, "default":"banner",
				"help":"set FIGlet font that the digital clock should display (defaults to banner)"}),
				
			(("-tc", "--topcolor"), {"type":int, "default":0,
				"help":"set color for top text (default 15)"}),
			
			(("-bc", "--bottomcolor"), {"type":int, "default":0,
				"help":"set color for bottom text (default 15)"}),
			
			(("-ts", "--topstrf"), {"type":str, "default":"%I:%M:%S %p",
				"help":"string to be passed to time.strftime() for the top text (default \"%%I:%%M:%%S %%p\")"}),
			
			(("-bs", "--bottomstrf"), {"type":str, "default":"%A, %B %d, %Y",
				"help":"string to be passed to time.strftime() for the bottom text (default \"%%A, %%B %%d, %%Y\")"}),
			
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"run in screensaver mode (press any key to exit)"}),]]

def run(args):
	curses.wrapper(main,args)


#!/usr/bin/env python3
import random
import time
import curses
import argparse
import sys
from tscrnsvr.shared import ScreenSpace

__VERSION__ = "v0.1.1"

# bounce.py
# Bounce text screensaver.
# by illinoisjackson: https://github.com/illinoisjackson

if sys.version_info <= (3,0):
	_PY2 = 1
else:
	_PY2 = 0

class BounceText:
	def __init__(self,win,text,colors):
		self.win	= win
		self.pos = [10,10]
		self.text	= text
		self.colors = colors
		self.motion = [random.choice([-1,1]),random.choice([-1,1])]
		self.lln = 0
		for line in self.text.split("\n"):
			if len(line) > self.lln:
				self.lln = len(line)
		self.size = [self.lln,len(self.text.split("\n"))]
	def render(self,screenspace):
		for ln, line in enumerate(self.text.split("\n")):
			for cn, char in enumerate(line):
				screenspace.addchar(self.pos[0]+cn,self.pos[1]+ln,ord(char),self.colors)
		for xu in range(self.pos[0]-1, \
						self.pos[0]+self.size[0]+1):
			for yu in range(self.pos[1]-1, \
							self.pos[1]+self.size[1]+1):
				if xu < 0:
					self.motion[0] = 1
				if xu >= self.win.getmaxyx()[1]:
					self.motion[0] = -1
				if yu < 0:
					self.motion[1] = 1
				if yu >= self.win.getmaxyx()[0]:
					self.motion[1] = -1
		self.pos[0] += self.motion[0]
		self.pos[1] += self.motion[1]
	
def main(stdscr,args):
	STARTTIME = time.time()
	TICK = 0
	SCSP = ScreenSpace(stdscr)
	bt	 = BounceText(stdscr,args.string,args.color)
	try:
		while True:
			bt.render(SCSP)
			deb = []
			if (time.time()-STARTTIME) < 2:
				if not args.screensaver:
					deb.append("Press Q or CTRL-C to quit.")
				if _PY2:
					deb.append("NOTE: Colors do not work properly in python2.")
					deb.append("Try running in python3 to get colors working.")
			if args.debug == True:
				deb.append("bounce.py %s" % __VERSION__)
				deb.append("%s\tframes, delay %s, color %s" % (str(TICK),str(args.delay),args.color))
			SCSP.render(debug=deb)
			time.sleep(args.delay)
			if args.screensaver:
				if SCSP.win.getch() != -1:
					SCSP.destroy()
					sys.exit()
			else:
				if SCSP.win.getch() == ord("q"):
					SCSP.destroy()
					sys.exit()
			TICK += 1
	except KeyboardInterrupt:
		SCSP.destroy()
		sys.exit()

def get_args():
	return [
			"bounce",
			"Bounce text around your terminal.",
			[(("-d", "--delay"), {"type":float, "default":0.1,
				"help":"set delay between frames in seconds (default 0.1)"}),
			
			(("-c", "--color"), {"type":int, "default":15,
				"help":"set color of text (default 15)"}),
			
			(("-g", "--debug"), {"action":"store_true",
				"help":"show debug statistics in upper left corner"}),
			
			(("-s", "--screensaver"), {"action":"store_true",
				"help":"screensaver mode (press any key to leave)"}),
			
			(("-S", "--string"), {"type":str, "default":"test",
				"help":"set string for bounce"})]
			]

def run(args):
	curses.wrapper(main,args)
		